#!/usr/bin/env python
#-*- coding:utf-8 -*-

import smtpd
import asyncore
import sys

class CustomFileSMTPServer(smtpd.SMTPServer):

	def __init__(*args, **kwargs):
		smtpd.SMTPServer.__init__(*args, **kwargs)


	def set_ofile(self,name):
		self.ofile = open(name, 'w')

	def process_message(self, peer, mailfrom, rcpttos, data):
		out = self.ofile
		out.write('Receiving message from: %s\n' % peer[0])
		out.write('Message addressed from: %s\n' % mailfrom)
		out.write('Message addressed to  : %s\n' % rcpttos)
		out.write('Message length        : %s\n' % len(data))
		out.close()
		return


def main():
	if len(sys.argv) == 2:
		ofile = sys.argv[1]
	else:
		ofile = '/dev/null'
	server = CustomFileSMTPServer(('127.0.0.1', 1555), None)
	server.set_ofile(ofile)
	asyncore.loop()

if __name__ == '__main__':
	sys.exit(main())



#!/usr/bin/env python
#-*- coding:utf-8 -*-

from os import environ as env
import tempfile
import shutil
import os
from ConfigParser import SafeConfigParser
import getpass
import socket
import sys
import mmailer
from mmail.mmailer import MMailer
import subprocess
from subprocess import PIPE, Popen, call
from StringIO import StringIO



class Test_MMailer(object):

	def setup(self):
		self.old_env_home = env['HOME']
		self.temp_dir = tempfile.mkdtemp()
		env['HOME'] = self.temp_dir
		self.config_dir = os.path.join(\
			self.temp_dir, '.config', 'mmailer')
		self.config_path = os.path.join(self.config_dir, 'config')
		self.projects_dir = os.path.join(self.config_dir, 'projects')
		self.last_call = None

	def teardown(self):
		env['HOME'] = self.old_env_home
		shutil.rmtree(self.temp_dir)

	def test_first_start_config_file_created(self):
		mmailer.main()
		assert os.path.isfile(self.config_path)

	def get_default_config_scheme(self):
		uname = getpass.getuser()
		host = socket.gethostname()
		citems = []
		citems.append(('server', host))
		citems.append(('port', '1025'))
		citems.append(('esmtp', 'no'))
		citems.append(('tls', 'no'))
		citems.append(('user', uname))
		citems.append(('password', ''))
		citems.append(('email', uname + "@" + host))
		return citems

	def get_testserver_config_scheme(self):
		uname = getpass.getuser()
		host = socket.gethostname()
		citems = []
		citems.append(('server', 'localhost'))
		citems.append(('port', '1555'))
		citems.append(('esmtp', 'False'))
		citems.append(('tls', 'False'))
		citems.append(('user', ''))
		citems.append(('password', ''))
		citems.append(('email', 'x' + "@" + 'localhost'))
		return citems

	def test_first_start_config_file_filled(self):
		mmailer.main()
		citems = self.get_default_config_scheme()
		config = SafeConfigParser()
		config.read(self.config_path)
		assert config.has_section('Mail')
		assert config.items('Mail') == citems

	def test_2nd_start_dont_overwrite_config(self):
		try:
			mmailer.main()
		except:
			pass
		mod_time = os.path.getmtime(self.config_path)
		try:
			mmailer.main()
		except:
			pass
		mod_time2 = os.path.getmtime(self.config_path)
		assert mod_time == mod_time2

	def is_project_selected_in_config(self, target=None):
		config = SafeConfigParser()
		config.read(self.config_path)
		if target != None:
			assert config.has_section('Project')
			assert config.has_option('Project', 'selected')
			assert config.get('Project', 'selected') == target, \
				"<%s> in config should be <%s>" \
					% (config.get('Project', 'selected'), target)
		else:
			assert not config.has_section('Project')

	def test_start_no_arg_right_output(self):
		error_screen = []
		error_screen.append("usage: mmailer [-h]")
		error_screen.append("               " \
			+ "{new,list,version,select,edit,edit-table,remove,config,send}")
		error_screen.append("               ...")
		error_screen.append("mmailer: error: too few arguments")
		try:
			output = subprocess.check_output('./mmailer',
				stderr=subprocess.STDOUT)
		except subprocess.CalledProcessError as cp:
			output = cp.output.splitlines()
		assert output == error_screen, "\n<%s>\nvs\n<%s>" \
			% (output, error_screen)

	def test_list_no_proj(self):
		output = self.mm_list()
		output_target = str("No project availible, create it with"
			+" the new command!\n")
		assert output == output_target

	def mm_list(self):
		return subprocess.check_output(['./mmailer', 'list'])

	def mm_remove(self):
		p = subprocess.Popen(['./mmailer', 'remove'],\
			stdin=PIPE, stdout=PIPE)
		question = p.communicate('yes')[0]
		assert question == 'Are you shure? [yes/NO] :', question
		try:
			p.communicate()
		except:
			pass
		assert p.poll() == 0
		return


	def mm_select(self, project):
		p = subprocess.call(['./mmailer', 'select', project])


	def mm_send(self):
		p = subprocess.Popen(['./mmailer', 'send'],\
			stdin=PIPE, stdout=PIPE)
		question = p.communicate('yes')[0]
		try:
			p.communicate()
		except:
			pass
		assert p.poll() == 0
		return

	def mm_create_project(self, name):
		subprocess.call(['./mmailer', 'new', name])
		return os.path.join(self.projects_dir, name)

	def test_command_list_filled(self):
		"adds some projects and checks output of the command <list>"
		prj_added = []
		for p in ['B-Project', 'A-Project', 'C-Project']:
			self.mm_create_project(p)
			prj_added.append(p)
			prj_added.sort()
			pa_lines=''
			for pa in prj_added:
				if pa == 'B-Project':
					pa_lines += "%s *\n" % (pa)
				else:
					pa_lines += "%s\n" % (pa)
			prj_str = 'Projects:\n---------\n%s\n' % (pa_lines)
			assert self.mm_list() == prj_str, '\n' + self.mm_list()+ prj_str

	def test_command_new_works(self):
		list_projects = []
		list_projects.append('Project "Test" created!')
		proj_dir = self.mm_create_project('Test')
		assert os.path.exists(proj_dir)
		assert os.path.isdir(proj_dir)
		self.is_project_selected_in_config('Test')

	def test_command_new_2nd_start_dont_override_proj(self):
		proj_dir = self.mm_create_project('project_test')
		mod_time = os.path.getmtime(proj_dir)
		self.mm_create_project('project_test')
		mod_time2 = os.path.getmtime(proj_dir)
		assert mod_time == mod_time2

	def test_command_remove_proj_dir(self):
		'checks if the remove command removes (only) the proj dir'
		proj1_dir = self.mm_create_project('Test')
		proj2_dir = self.mm_create_project('Test2')
		self.mm_remove()
		assert not os.path.exists(proj1_dir), proj1_dir
		assert os.path.exists(proj2_dir)

	def test_command_remove_updates_selected_proj(self):
		projects = ['B-Project', 'C-Project', 'A-Project']
		for p in projects:
			self.mm_create_project(p)
		self.is_project_selected_in_config(projects[0])
		self.mm_remove()
		self.is_project_selected_in_config(projects[2])
		self.mm_remove()
		self.is_project_selected_in_config(projects[1])
		self.mm_remove()
		self.is_project_selected_in_config(None)

	def save_call(self, call_dict):
		self.last_call = call_dict

	def test_command_edit(self):
		proj_dir = self.mm_create_project('Test')
		env['EDITOR'] = 'gedit'
		sys.argv = ['mmailer', 'edit']
		subprocess_call_save = subprocess.call
		subprocess.call = self.save_call
		assert mmailer.main(sys.argv) == 0
		subprocess.call = subprocess_call_save
		files = ['mail.txt', 'subject.txt', 'attachments.txt']
		abs_files = []
		for f in files:
			abs_file = os.path.join(proj_dir, f)
			#assert os.path.isfile(abs_file)
			abs_files.append(abs_file)
		assert self.last_call == ['gedit', abs_files[0], \
			abs_files[1], abs_files[2]], self.last_call

	def create_with_sample_files(self, sample_dir, proj_dir, files=None):
		if files == None:
			files = ['mail.txt', 'subject.txt']
		for f in files:
			shutil.copy(os.path.join('tests','samples', sample_dir, f), proj_dir)


	def test_command_edit_table(self):
		proj_dir = self.mm_create_project('Test')
		self.create_with_sample_files("", proj_dir)
		env['EDITOR'] = 'gedit'
		sys.argv = ['mmailer', 'edit-table']
		subprocess_call_save = subprocess.call
		subprocess.call = self.save_call
		abs_file = os.path.join(proj_dir, 'keys.csv')
		assert mmailer.main(sys.argv) == 0
		subprocess.call = subprocess_call_save
		assert os.path.exists(abs_file), os.path.join(proj_dir, 'keys.txt')
		assert self.last_call == ['gedit', abs_file], self.last_call
		import filecmp
		target = os.path.join('tests','samples', 'keys-empty.csv')
		assert filecmp.cmp(target, abs_file)

	def test_command_select(self):
		projects = ['A-Project', 'B-Project', 'C-Project']
		for p in projects:
			self.mm_create_project(p)
		self.is_project_selected_in_config(projects[0])
		self.mm_select('C-Project')
		self.is_project_selected_in_config(projects[2])

	def test_command_select_non_existend(self):
		projects = ['A-Project', 'B-Project']
		for p in projects:
			self.mm_create_project(p)
		self.is_project_selected_in_config(projects[0])
		self.mm_select('D-Project')
		self.is_project_selected_in_config(projects[0])

	def mm_config_fill(self, port=1555):
		questions = ['SMTP Server Settings', '\n\n\n', \
				'Server :', 'Port [587] :', 'esmtp [YES/no] :', \
				'tls [YES/no] :', 'Whats your login :',  \
				'Whats your password :', 'Whats your email address :', \
				'try to connect!\n']
		answers = ['localhost', str(port), 'no', 'no', '', '', 'x@localhost']
		sys.stdin = x_in = StringIO('\n'.join(answers))
		sys.stdout = x_out = StringIO()
		m = MMailer()
		mod_time = os.path.getmtime(self.config_path)
		m.command_config(args=['mmailer', '	config'])
		mod_time2 = os.path.getmtime(self.config_path)
		config_changed = not (mod_time == mod_time2)
		sys.stdin = sys.__stdin__
		sys.stdout = sys.__stdout__
		#assert "".join(questions) == x_out.getvalue(), \
		#	'\n[%s] != \n[%s]' % ("".join(questions), x_out.getvalue())
		return config_changed


	def test_command_config_with_invalid_smtp(self):
		config_changed = self.mm_config_fill()
		citems = self.get_default_config_scheme()
		config = SafeConfigParser()
		config.read(self.config_path)
		assert config.has_section('Mail')
		assert config.items('Mail') == citems, \
			"\n[%s]\n[%s]" % (config.items('Mail'), citems)


	def test_command_config_with_valid_smtp(self):
		# START SMTPD
		p1 = Popen(['./tests/smtp_server.py'], stdin=PIPE)
		import time
		time.sleep(0.1)
		config_changed = self.mm_config_fill()
		p1.terminate()
		p1.wait()
		citems = self.get_testserver_config_scheme()
		config = SafeConfigParser()
		config.read(self.config_path)
		assert config.has_section('Mail')
		assert config.items('Mail') == citems, \
			"\n[%s]\n[%s]" % (config.items('Mail'), citems)

	def test_command_send(self):
		# START SMTPD
		tfile = tempfile.NamedTemporaryFile()
		tfile_name = tfile.name
		tfile.close()
		p1 = Popen(['./tests/smtp_server.py', tfile_name], stdin=PIPE)
		citems = self.get_testserver_config_scheme()
		proj_dir = self.mm_create_project('Test')
		config = SafeConfigParser()
		config.read(self.config_path)
		for item in citems:
			config.set('Mail', item[0], item[1])
		files = ['mail.txt', 'subject.txt', 'keys.csv', 'attachments.txt']
		self.create_with_sample_files("", proj_dir, files)
		config.write(open(self.config_path, 'w'))
		sys.stdin = x_in = StringIO('\n')
		sys.stdout = x_out = StringIO()
		m = MMailer()
		m.command_send(args=['mmailer', 'send'])
		sys.stdin = sys.__stdin__
		sys.stdout = sys.__stdout__
		p1.terminate()
		p1.wait()
		of = open('tests/samples/send_output.txt','r')
		sto_str = of.read()
		sto = sto_str.decode('UTF-8')
		# exclude first line from compare,
		# because the date changes each test run
		assert x_out.getvalue() == sto,"\n\n[%s]\n\n[%s]" \
				% (sto, x_out.getvalue())
		import filecmp
		assert filecmp.cmp(tfile_name, 'tests/samples/sended.txt')




#!/usr/bin/env python

import versioneer
versioneer.versionfile_source = 'mmail/_version.py'
versioneer.versionfile_build = 'mmail/_version.py'
versioneer.tag_prefix = ''
versioneer.parentdir_prefix = 'mmailer-'
import os

#from setuptools import setup
from distutils.core import setup


setup(name='mmailer',
			version=versioneer.get_version(),
			cmdclass=versioneer.get_cmdclass(),
			description='Serial/Mass mail tool',
			author='Stefan Huchler',
			author_email='s.huchler@gmail.com',
			url='https://github.com/spiderbit/mmailer',
			license='LICENSE',
			long_description=open('README').read(),
			packages = ['mmail'],
			py_modules = ['versioneer'],
			classifiers=[
					'Development Status :: 5 - Production/Stable',
					'Environment :: Console',
					'Intended Audience :: End Users/Desktop',
					'Intended Audience :: Developers',
					'Intended Audience :: System Administrators',
					'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
					'Operating System :: MacOS :: MacOS X',
					'Operating System :: Microsoft :: Windows',
					'Operating System :: POSIX',
					'Programming Language :: Python',
					'Topic :: Communications :: Email',
					'Topic :: Office/Business',
					],
			scripts=['mmailer']
			)



#!/usr/bin/env python
#-*- coding:utf-8 -*-

from string import Template
import re
import sys
import os
import smtplib

from ConfigParser import SafeConfigParser
from email.MIMEBase import MIMEBase
from email import Encoders
from email.MIMEMultipart import MIMEMultipart
from smtplib import SMTP
from email.MIMEText import MIMEText
from email.Header import Header
from email.Utils import parseaddr, formataddr
import subprocess
import os.path
import csv
import getpass
import socket
from os import environ as env
import argparse
import shutil

def ask(question, typ, default=None):
	value = None
	while(value == None):
		done = True
		if typ == 'bool':
			choice = re.sub(default, default.upper(), '[yes/no]')
			user_input = raw_input('%s %s :' % (question, choice))
			user_input = user_input.lower()
			if user_input == 'yes' or user_input == '':
				value = True
			elif user_input == 'no':
				value = False
		elif typ == 'string':
			if default == None or default == '':
				user_input = raw_input('%s :' % (question))
				value = user_input
			else:
				user_input = raw_input('%s [%s] :' % (question, default))
				if user_input == '':
					value = default
				else:
					value = user_input
		elif typ == 'int':
			user_input = raw_input('%s [%s] :' % (question, default))
			if user_input == '':
				value = default
			else:
				try:
					value = int (user_input)
				except:
					pass
		if value == None:
			print "Please type in one of the choices."
	return value


class SMTP(object):
	def __init__(self):
		self.cm = ConfigManager()
		self.load_config()

	def load_config(self):
		config = self.cm.config
		self.smtp_server = config.get('Mail', 'server')
		self.port = config.get('Mail', 'port')
		self.esmtp = config.getboolean('Mail', 'esmtp')
		self.tls = config.getboolean('Mail', 'tls')
		self.user = config.get('Mail', 'user')
		self.password = config.get('Mail', 'password')
		self.mail_from = config.get('Mail', 'email')
		self.connected = False

	def connect(self):
		try:
			self.server = smtplib.SMTP(self.smtp_server, self.port)
			if self.esmtp:
				self.server.ehlo()
			if self.tls:
				self.server.starttls()
			if self.user != '':
				self.server.login(self.user, self.password)
			self.connected = True
		except:
			self.connected = False


	def ask_settings(self):
		# should be a mmailer config method
		home = env['HOME']
		config = self.cm.config
		mail = None
		print 'SMTP Server Settings\n\n'
		value = str(ask('Server', 'string'))
		self.cm.config.set('Mail', 'server', value)
		value = str(ask('Port', 'int', '587'))
		self.cm.config.set('Mail', 'port', value)
		value = str(ask('esmtp', 'bool', 'yes'))
		self.cm.config.set('Mail', 'esmtp', value)
		value = str(ask('tls', 'bool', 'yes'))
		self.cm.config.set('Mail', 'tls', value)
		value = str(ask('Whats your login', 'string'))
		self.cm.config.set('Mail', 'user', value)
		value = str(ask('Whats your password', 'string'))
		self.cm.config.set('Mail', 'password', value)
		value = str(ask('Whats your email address',
			'string', config.get('Mail', 'user')))
		self.cm.config.set('Mail', 'email', value)
		self.load_config()
		print "try to connect!"
		self.connect()
		mail_txt= "Test mail from the tool mmailer" \
			" (https://github.com/spiderbit/mmailer) tool to see if the" \
			" given smtp server works"
		tested = True
		try:
			self.server.sendmail(self.mail_from, self.mail_from, mail_txt)
		except:
			tested = False
		if tested:
			self.cm.write()
		else:
			print ("could not connect - restart and try again!")


	def quit(self):
		if self.connected:
			self.server.quit()


class Mail (object):

	def __init__(self, files=[], smtp=None):
		self.files = files
		self.smtp = smtp
		self.approved = False


	def show(self):
		# We must choose the body charset manually
		for body_charset in 'US-ASCII', 'ISO-8859-1', 'UTF-8':
			try:
				self.body.encode(body_charset)
			except UnicodeError:
				pass
			else:
				break
		body_msg = MIMEText(self.body.encode(body_charset), \
			'plain', body_charset)
		print self.body
		print "send to: %s" % self.recipient
		print "subject: <%s>" % self.subject


	def send(self):
		"""Send an email.

		All arguments should be Unicode strings (plain ASCII works as well).

		Only the real name part of sender and recipient addresses may contain
		non-ASCII characters.

		The email will be properly MIME encoded and delivered though SMTP to
		localhost port 25.  This is easy to change if you want something different.

		The charset of the email will be the first one out of US-ASCII, ISO-8859-1
		and UTF-8 that can represent all the characters occurring in the email.
		"""

		if not self.approved:
			return False

		# Header class is smart enough to try US-ASCII, then the charset we
		# provide, then fall back to UTF-8.
		header_charset = 'ISO-8859-1'

		# We must choose the body charset manually
		for body_charset in 'US-ASCII', 'ISO-8859-1', 'UTF-8':
			try:
				self.body.encode(body_charset)
			except UnicodeError:
				pass
			else:
				break

		# Split real name (which is optional) and email address parts
		sender_name, sender_addr = parseaddr(self.sender)
		recipient_name, recipient_addr = parseaddr(self.recipient)

		# We must always pass Unicode strings to Header, otherwise it will
		# use RFC 2047 encoding even on plain ASCII strings.
		sender_name = str(Header(unicode(sender_name), header_charset))
		recipient_name = str(Header(unicode(recipient_name), header_charset))

		# Make sure email addresses do not contain non-ASCII characters
		sender_addr = sender_addr.encode('ascii')
		recipient_addr = recipient_addr.encode('ascii')

		# Create the message ('plain' stands for Content-Type: text/plain)
		msg = MIMEMultipart()
		body_msg = MIMEText(self.body.encode(body_charset), 'plain', body_charset)
		msg['From'] = formataddr((sender_name, sender_addr))
		msg['To'] = formataddr((recipient_name, recipient_addr))
		msg['Subject'] = Header(unicode(self.subject), header_charset)
		msg["Content-type"] = "text/html;charset=utf-8"
		msg.attach(body_msg)
		for f in self.files:
			part = MIMEBase('application', "octet-stream")
			part.set_payload( open(f,"rb").read() )
			Encoders.encode_base64(part)
			part.add_header('Content-Disposition', \
				'attachment; filename="%s"' % os.path.basename(f))
			msg.attach(part)
		# Send the message via SMTP
		self.smtp.connect()
		self.smtp.server.sendmail(self.sender, self.recipient, msg.as_string())
		return True



class ConfigManager (object):

	def __init__(self):
		uname = getpass.getuser()
		host = socket.gethostname()
		home = env['HOME']
		self.config_dir = os.path.join(home, '.config', 'mmailer')
		self.config_path = os.path.join(self.config_dir, 'config')
		config = self.config = SafeConfigParser()
		if os.path.isfile(self.config_path):
			self.read()
		else:
			config.add_section('Mail')
			config.set('Mail', 'server', host)
			config.set('Mail', 'port', '1025')
			config.set('Mail', 'esmtp', 'no')
			config.set('Mail', 'tls', 'no')
			config.set('Mail', 'user', uname)
			config.set('Mail', 'password', '')
			config.set('Mail', 'email', uname + "@" + host)
			self.write()

	def write(self):
		if not os.path.exists(self.config_dir):
			os.makedirs(self.config_dir)
		with open(self.config_path, 'wb') as config_file:
			self.config.write(config_file)

	def read(self):
		self.config.read(self.config_path)

	def set_active_project(self, name):
		if not self.config.has_section('Project'):
			self.config.add_section('Project')
		self.config.set('Project', 'selected', name)
		self.write()

	def remove_section_project(self):
		if self.config.has_section('Project'):
			self.config.remove_section('Project')
		self.write()

	def get_active_project(self):
		if self.config.has_section('Project'):
			return self.config.get('Project', 'selected')
		return None



class Project (object):

	def __init__(self, name):
		self.name = name
		home = env['HOME']
		config_dir = os.path.join(home, '.config', 'mmailer')
		self.projects_dir = os.path.join(config_dir, 'projects')
		self.proj_dir = os.path.join(self.projects_dir, name)
		self.attachments=[]
		self.template_files=dict()
		self.template_files['mail'] = \
			os.path.join(self.proj_dir, 'mail.txt')
		self.template_files['subject'] = \
			os.path.join(self.proj_dir, 'subject.txt')
		self.keys = []
		self.cm = ConfigManager()
		self.mails = []

	def create(self):
		if not os.path.exists(self.proj_dir):
			os.makedirs(self.proj_dir)

	def create_files(self):
		sys.exit("code commented out for reinspection")
		# overrides the files at the moment
		# maybe not neaded for edit command anyway
		for f in self.files():
			p = open(f, 'w')
			p.write('')
			p.close()

	def files(self):
		files = ['mail.txt', 'subject.txt', 'attachments.txt']
		abs_files = []
		for f in files:
			abs_files.append(os.path.join(self.proj_dir, f))
		return abs_files

	def substitude_file(self):
		sfile = os.path.join(self.proj_dir, 'keys.csv')
		return sfile

	def remove(self):
		shutil.rmtree(self.proj_dir)

	def load_keys(self):
		template_files = self.template_files
		for k,v in template_files.iteritems():
			#if not v in os.listdir('.'):
			#	print "need a file %s in this directory as %s-template!" %(v,k)
			#	sys.exit()
			#else:
			fp = open(v, 'rb')
			plaintext = fp.read()
			tmp_keys = re.findall('\$([a-zA-Z_]+)', plaintext)
			self.keys.extend(tmp_keys)
		self.keys = list(set(self.keys)) # to remove double entries
		self.keys.append('email')

	def create_table(self):
		abs_files = self.files()
		self.load_keys()
		keys = self.keys
		csv_fn = os.path.join(self.proj_dir, 'keys.csv')
		if not os.path.exists(csv_fn):
			with open(csv_fn, 'wb') as csv_file:
				writer = csv.DictWriter(csv_file, keys)
				writer.writeheader()
		#afile = os.path.join(self.proj_dir, 'attachments.txt')

		#if not os.path.exists(afile):
		#	print 'no %s file found!', (afile)
		#	print ("If you want attachements create it "
		#		"and place one file location per line in it!")
		#	value = ask('Should I exit mmailer and create'
		#		' that file empty for you?', 'bool', 'yes')
		#	if value == 'true':
		#		open(afile, 'w+')
		#		sys.exit(0)
		#else:

	def load_attachments(self):
		afile = os.path.join(self.proj_dir,'attachments.txt')
		with open(afile, 'rb') as afile:
			for row in afile:
				if os.path.isfile(row[:-1]):
					self.attachments.append(row[:-1])
				else:
					sys.exit("one of your attachments \
					dont exist or is no file: <%s>" % (row[:-1]))
		return

	def send(self):
		self.load_keys()
		keys = self.keys
		config = self.cm.config
		self.load_attachments()
		smtp = SMTP()
		with open(self.substitude_file(), 'rb') as csv_file, \
		open(self.template_files['mail'], 'rb') as mail_template,\
		open(self.template_files['subject'], 'rb') as subject_template:
			reader = csv.DictReader(csv_file, keys)
			output = Template(mail_template.read())
			reader.next()
			output_subject = Template(subject_template.readline()[:-1])
			count = 0

			for row in reader:
				mail = Mail(self.attachments, smtp)
				msg_only = output.substitute(row)
				mail.subject = unicode(output_subject.substitute(row), \
					encoding='UTF-8')
				mail_to = row['email']
				mail.body = unicode(msg_only, encoding='UTF-8')
				mail.recipient = unicode(mail_to, encoding='UTF-8')
				mail.sender = unicode(config.get('Mail', 'email'), \
					encoding='UTF-8')
				mail.show()
				mail.approved = ask('Look over the mail is it all right?'\
					' should I send it for you?', 'bool', 'yes')
				self.mails.append(mail)

		smtp.connect()
		for mail in self.mails:
			if mail.send():
				count+=1
		print "did send %s mail[s]!" % (count)
		smtp.quit()

class ProjectList (object):

	def __init__(self):
		self.projects = dict()
		self.cm = ConfigManager()
		home = env['HOME']
		config_dir = os.path.join(home, '.config', 'mmailer')
		self.projects_dir = os.path.join(config_dir, 'projects')
		if not os.path.exists(self.projects_dir):
			os.makedirs(self.projects_dir)
		self.search()

	def projects_sorted(self):
		keylist = self.projects.keys()
		keylist.sort()
		return keylist

	def search(self):
		files = os.listdir(self.projects_dir)
		for f in files:
			if os.path.isdir(os.path.join(self.projects_dir, f)):
				self.projects[f] = Project(f)

	def add_project(self, name):
		if len(self.projects) == 0:
			self.cm.set_active_project(name)
		p = Project(name)
		p.create()
		self.projects[name] = p

	def remove(self, name):
		self.projects[name].remove()
		del self.projects[name]
		if len (self.projects) > 0:
			new_default = self.projects_sorted()[0]
			self.cm.set_active_project(new_default)
		else:
			self.cm.remove_section_project()

	def get_active_project(self):
		return self.cm.get_active_project()


	def set_active_project(self, name):
		return self.cm.set_active_project(name)


class MMailer (object):

	def __init__(self):
		self.cm = ConfigManager()

	def command_new(self, args):
		name = args.proj_name
		pl = ProjectList()
		if not name in pl.projects.keys():
			pl.add_project(name)
		return

	def command_list(self, args):
		pl = ProjectList()
		if len(pl.projects) == 0:
			print "No project availible, create it with the new command!"
		else:
			p_names = pl.projects_sorted()
			pa_lines=''
			for pa in p_names:
				selected_prj = pl.cm.config.get('Project', 'selected')
				if pa == selected_prj:
					pa_lines += "%s *\n" % (pa)
				else:
					pa_lines += "%s\n" % (pa)
			prj_str = 'Projects:\n---------\n%s' % (pa_lines)
			print prj_str

	def command_edit(self, args):
		pl = ProjectList()
		active_project = pl.get_active_project()
		if active_project != None:
			p = pl.projects[active_project]
			#p.create_files()
			retvalue = subprocess.call( \
				[env['EDITOR']] + p.files())
			if retvalue == 0:
				pass
		return

	def command_edit_table(self, args):
		pl = ProjectList()
		active_project = pl.get_active_project()
		if active_project != None:
			p = pl.projects[active_project]
			p.create_table()
			retvalue = subprocess.call(
				[env['EDITOR'], p.substitude_file()])
			if retvalue == 0:
				pass
		return

	def command_remove(self, args):
		pl = ProjectList()
		active_project = pl.get_active_project()
		if active_project != None:
			confirmed = ask('Are you shure?', 'bool', 'no')
			if confirmed:
				pl.remove(active_project)
		return

	def command_select(self, args):
		name = args.proj_name
		pl = ProjectList()
		if name in pl.projects.keys():
			pl.set_active_project(name)
		return

	def command_send(self, args):
		pl = ProjectList()
		active_project = pl.get_active_project()
		if active_project != None:
			p = pl.projects[active_project]
			p.send()


	def command_config(self, args):
		smtp = SMTP()
		smtp.ask_settings()

	def command_version(self, args):
		from _version import get_versions
		print get_versions()['version']

def args_config():
	m = MMailer()
	parser = argparse.ArgumentParser(
		description='Serial mail managing tool.')
	subparsers = parser.add_subparsers()
	new_parser = subparsers.add_parser('new')
	new_parser.add_argument('proj_name')
	new_parser.set_defaults(func=m.command_new)
	list_parser = subparsers.add_parser('list')
	list_parser.set_defaults(func=m.command_list)
	version_parser = subparsers.add_parser('version')
	version_parser.set_defaults(func=m.command_version)
	select_parser = subparsers.add_parser('select')
	select_parser.add_argument('proj_name')
	select_parser.set_defaults(func=m.command_select)
	edit_parser = subparsers.add_parser('edit')
	edit_parser.set_defaults(func=m.command_edit)
	edit_table_parser = subparsers.add_parser('edit-table')
	edit_table_parser.set_defaults(func=m.command_edit_table)
	remove_parser = subparsers.add_parser('remove')
	remove_parser.set_defaults(func=m.command_remove)
	config_parser = subparsers.add_parser('config')
	config_parser.set_defaults(func=m.command_config)
	send_parser = subparsers.add_parser('send')
	send_parser.set_defaults(func=m.command_send)
	new_parser.description = "creates a new project"
	version_parser.description = "prints out the version of mmailer"
	list_parser.description = "lists all projects"
	select_parser.description = "selects a project"
	edit_parser.description = "edits the project files"
	edit_table_parser.description = "generates a table where each line \
		you fill will create a mail"
	remove_parser.description = "removes the active project"
	config_parser.description = "asks you for you smtp-server settings"
	send_parser.description = "sends the emails generated from your \
		current project"
	args = parser.parse_args(sys.argv[1:])
	if 'func' in args:
		args.func(args)


def main(args=None):

	args_config()
	return 0



if __name__ == '__main__':
	sys.exit(main(args=sys.argv))
